#!/usr/bin/env python

## standard imports
import argparse
import os
import sys
from utils import unprocessing
import imageio
import numpy as np
import imageio
from pidng.core import RAW2DNG, DNGTags, Tag


## Data
parser = argparse.ArgumentParser(description="I don't need it")
parser.add_argument("--black_level", type=int           , default=240     ,help='who needs help???')
parser.add_argument("--white_level", type=int           , default=4095    ,help='who needs help???')
parser.add_argument("--gamma"      , type=float         , default=2.2     ,help='who needs help???')
parser.add_argument("--camera"     , type=str           , default='convex',help='who needs help???')
parser.add_argument("--red_gain"   , type=float         , default=2.15    ,help='who needs help???')
parser.add_argument("--blue_gain"  , type=float         , default=1.7     ,help='who needs help???')
parser.add_argument("--rgb_gain"   , type=float         , default=1.25    ,help='who needs help???')
#parser.add_argument("--add_noise"  , action='store_true'                  ,help='who needs help???')
parser.add_argument("--add_noise"  , type=str                             ,help='who needs help???')
parser.add_argument("--a"          , type=float         , default=8       ,help='who needs help???')
parser.add_argument("--b"          , type=float         , default=-1900   ,help='who needs help???')

args = parser.parse_args()

args.add_noise = args.add_noise.lower() == "true"

# Input path for the image
img_path = "input_0.png"


## Scripts
img = imageio.imread(img_path)
bayer_out, CCM = unprocessing(img, args.camera, args.gamma, args.red_gain, args.blue_gain, args.rgb_gain)


m, M = bayer_out.min(), bayer_out.max()
bayer_out255= np.round(255*(bayer_out-m) / (M-m)).clip(0,255).astype(np.uint8)  #image displayed in the demo will be png 8 bits
bayer_outraw = bayer_out * (args.white_level - args.black_level) + args.black_level

#Add noise
if args.add_noise:
    noisy_bayer = bayer_outraw + np.sqrt( (args.a*bayer_outraw + args.b).clip(0, np.inf) ) * np.random.randn(*bayer_outraw.shape)
    m, M = noisy_bayer.min(), noisy_bayer.max()
    noisy_bayer255 = np.round(255*(noisy_bayer-m) / (M-m)).clip(0,255).astype(np.uint8) #image will be display as png-8


bayer_outraw = np.round(bayer_outraw).clip(args.black_level, args.white_level).astype(np.uint16)
if args.add_noise:
    noisy_bayer  = np.round(noisy_bayer).clip(0, args.white_level).astype(np.uint16)

width = bayer_outraw.shape[1]
height = bayer_outraw.shape[0]

if width%2==1:
    width = width-1
if height%2==1:
    height = height-1

#convert CCM into the PiDNG format
ccm = np.zeros((9,2))
ccm[0] = [CCM[0,0]*10000, 10000]
ccm[1] = [CCM[0,1]*10000, 10000]
ccm[2] = [CCM[0,2]*10000, 10000]
ccm[3] = [CCM[1,0]*10000, 10000]
ccm[4] = [CCM[1,1]*10000, 10000]
ccm[5] = [CCM[1,2]*10000, 10000]
ccm[6] = [CCM[2,0]*10000, 10000]
ccm[7] = [CCM[2,1]*10000, 10000]
ccm[8] = [CCM[2,2]*10000, 10000]
ccm = np.int16(ccm)

as_shot_color_balance = np.array([[10000*args.red_gain*args.rgb_gain, 10000], [10000*args.rgb_gain, 10000], [10000*args.blue_gain*args.rgb_gain, 10000]])
as_shot_color_balance = np.uint16(as_shot_color_balance)

# set DNG tags.
t = DNGTags()
t.set(Tag.ImageWidth, width)
t.set(Tag.ImageLength, height)
t.set(Tag.TileWidth, width)
t.set(Tag.TileLength, height)
t.set(Tag.BitsPerSample, 12)
t.set(Tag.CFARepeatPatternDim, [2,2])
t.set(Tag.CFAPattern, [0, 1, 1, 2])
t.set(Tag.BlackLevel, args.black_level)
t.set(Tag.WhiteLevel, args.white_level)
t.set(Tag.ColorMatrix1, ccm)
t.set(Tag.AsShotNeutral, as_shot_color_balance)
t.set(Tag.Model, args.camera)

# save to dng file.
RAW2DNG().convert(bayer_outraw[:height :width], tags=t, filename="output_clean_raw", path="")
if args.add_noise:
    RAW2DNG().convert(noisy_bayer[:height, :width], tags=t, filename="output_noisy_raw", path="")

#save images for the archive
imageio.imsave("output.png", bayer_out255)
if args.add_noise:
    imageio.imsave("output_noisy.png", noisy_bayer255)
